from synergy_file_reader.tools import (
		split_well_name, extract_channel,
		parse_time, parse_timestamp,
		row_iter,
		LineBuffer,
	)
from pytest import mark, raises

@mark.parametrize(
		"well, row, col",
		[
			( "A1"   , "A"  , 1   ),
			( "B12"  , "B"  , 12  ),
			( "AA1"  , "AA" , 1   ),
			( "ZX137", "ZX" , 137 ),
		]
	)
def test_split_well_name(well,row,col):
	assert split_well_name(well) == (row,col)

@mark.parametrize("well",["A","1","OD:600"])
def test_split_bad_well(well):
	with raises(ValueError):
		split_well_name(well)

@mark.parametrize(
		"string, name, channel",
		[
			( "Max V [OD:600]" , "Max V"     , "OD:600" ),
			( "Lagtime [YFP]"  , "Lagtime"   , "YFP"    ),
			( "R-Squared [CFP]", "R-Squared" , "CFP"    ),
		]
	)
def test_extract_channel(string,name,channel):
	assert extract_channel(string) == (name,channel)

@mark.parametrize("string",["Max V [OD:600","abc"])
def test_extract_bad_channel(string):
	with raises(ValueError):
		extract_channel(string)

@mark.parametrize(
		"string, seconds",
		[
			( "0:00:01",     1),
			( "0:01:00",    60),
			( "1:00:00",  3600),
			( "2:34:56",  9296),
		]
	)
def test_parse_time(string,seconds):
	assert parse_time(string) == seconds

@mark.parametrize(
		"string, number, seconds",
		[
			( "Time 1 (0:09:10)"   ,  1,   550 ),
			( "Time  98 (16:19:10)", 98, 58750 ),
		]
	)
def test_parse_timestamp(string,number,seconds):
	assert parse_timestamp(string) == (number,seconds)

@mark.parametrize("string",["Time 1 (0:09:10","abc"])
def test_parse_bad_timestamp(string):
	with raises(ValueError):
		parse_timestamp(string)

def test_row_iter():
	items = list(row_iter())
	assert items[0] == "A"
	assert items[25] == "Z"
	assert items[26] == "AA"
	assert items[28] == "AC"
	assert items[81] == "CD"

def test_LineBuffer():
	lb = LineBuffer(["","","a","b","","c","","","d","e","","","f",""])
	
	for line in lb:
		pass
	assert line==""
	
	for _ in range(3):
		for line,control in zip(lb,["a","b","","c","","","d"]):
			assert lb
			assert line==control
	
	with raises(ValueError):
		with lb as lb_iter:
			for line,control in zip(lb_iter,["a","b","","c"]):
				assert lb
				assert line==control
			raise ValueError
	
	with lb as lb_iter:
		for line,control in zip(lb_iter,["a","b","","c"]):
			assert lb
			assert line==control
	
	for line,control in zip(lb,["d","e","","","f"]):
		assert lb
		assert line==control
	lb.clear()
	
	assert not lb
	for line in lb:
		raise AssertionError





from synergy_file_reader import SynergyFile
import numpy as np
from matplotlib.pyplot import subplots

my_plate = SynergyFile("example_data.txt")[0]

baseline = np.percentile([ts[0] for ts in my_plate.data.values()],10)

channel = "OD:600"

fig,axess = subplots(
		len(my_plate.rows), len(my_plate.cols),
		figsize=(15,10),
		sharex="all", sharey="all"
	)

for axess_row,row in zip(axess,my_plate.rows):
	for axes,col in zip(axess_row,my_plate.cols):
		axes.plot(
				my_plate.times[channel]/3600,
				my_plate[row,col]-baseline,
			)

axes.set_yscale("log")
axes.set_xlim( 0,10 )
axes.set_ylim(1e-3,2)


for axes,col in zip(axess[-1],my_plate.cols):
	axes.set_xlabel("hours")
for axes,row in zip(axess[:,0],my_plate.rows):
	axes.set_ylabel(channel)


# Subplot labelling thanks to https://stackoverflow.com/a/25814386/2127008
pad = 20

for axes,col in zip(axess[0],my_plate.cols):
	axes.annotate(
			col,
			xy=(0.5,1), xycoords='axes fraction',
			xytext=(0,pad), textcoords='offset points',
			size='xx-large', ha='center', va='baseline'
		)

for axes,row in zip(axess[:,0],my_plate.rows):
	axes.annotate(
			row,
			xy=(0,0.5), xycoords=axes.yaxis.label,
			xytext=(-axes.yaxis.labelpad-pad,0), textcoords='offset points',
			size='xx-large', ha='center', va='center'
		)

fig.savefig("explicit_plot.pdf",bbox_inches="tight")





from synergy_file_reader import SynergyFile
from datetime import datetime
from pytest import mark
from os import path
import numpy as np

@mark.parametrize(
		  "filename,       temperature_ts",
	[
		( "column_matrix.txt", True  ),
		( "column_row.txt"   , True  ),
		( "column_column.txt", True  ),
		( "matrix_matrix.txt", False ),
		( "row_matrix.txt"   , True  ),
	])
def test_time_series(filename,temperature_ts):
	data = SynergyFile(path.join("time_series",filename))
	assert len(data)==1
	plate = data[0]
	
	assert plate.metadata["Software Version"] == (3,2,1)
	assert plate.metadata["Experiment File Path"] == r"C:\foo.xpt"
	assert plate.metadata["Protocol File Path"] == r"C:\bar.prt"
	assert plate.metadata["Plate Number"] == "Plate 1"
	assert plate.metadata["Reader Type"] == "Synergy H1"
	assert plate.metadata["Reader Serial Number"] == "18092726"
	assert plate.metadata["Reading Type"] == "Reader"
	assert plate.metadata["procedure"] == "foo\nbar\nquz"
	
	assert plate.metadata["datetime"] == datetime(2020,7,23,17,40,7)
	
	assert plate.channels == ["OD:600"]
	assert plate.rows == list("ABCDEFGH")
	assert plate.cols == list(range(1,13))
	
	times = plate.times["OD:600"]
	assert times[0] == 9*60+10
	assert times[-1] == 17*3600+29*60+10
	assert np.all( np.diff(times)==10*60 )
	
	if temperature_ts:
		temps = plate.temperatures["OD:600"]
		assert len(times) == len(temps)
		assert temps[ 0] == 30.0
		assert temps[-1] == 30.1
		assert temps[ 5] == 30.1
		assert plate.temperature_range == (30.0,30.1)
	else:
		assert plate.temperature_range == ( 0.0,30.1)
	
	assert plate["C12" ,"OD:600"][2] == 0.088
	assert plate["C",12,"OD:600"][2] == 0.088
	assert plate["C",12         ][2] == 0.088
	assert plate["C12"          ][2] == 0.088
	assert plate["c12" ,"OD:600"][2] == 0.088
	
	assert plate["A1" ][ 0] == 0.093
	assert plate["H12"][-1] == 0.099
	
	assert plate.results["Max V"]["D",2,"OD:600"] == -0.060
	assert plate.results["t at Max V"]["H11","OD:600"] == 5050
	assert plate.results["t at Max V"]["H11"] == 5050


@mark.parametrize(
		  "filename,     temperature_ts, separator",
	[
		( "column.txt"   ,   True ,       "\t" ),
		( "rowwise.txt"  ,   True ,       "\t" ),
		( "matrix.txt"   ,   False,       "\t" ),
		( "semicolon.txt",   True ,       ";"  ),
	])
def test_multiple_observables(filename,temperature_ts,separator):
	data = SynergyFile(
			path.join("multiple_observables",filename),
			separator
		)
	assert len(data)==1
	plate = data[0]
	
	assert plate.metadata["Software Version"] == (3,2,1)
	assert plate.metadata["Experiment File Path"] == r"C:\foo.xpt"
	assert plate.metadata["Protocol File Path"] == r"C:\bar.prt"
	assert plate.metadata["Plate Number"] == "Plate 1"
	assert plate.metadata["Reader Type"] == "Synergy H1"
	assert plate.metadata["Reader Serial Number"] == "18092726"
	assert plate.metadata["Reading Type"] == "Reader"
	assert plate.metadata["procedure"] == "foo\tbar\nquz"
	
	assert plate.metadata["datetime"] == datetime(2020,7,24,15,27,56)
	
	assert plate.channels == ["OD:600","485,528","530,580"]
	assert plate.rows == list("ABCDEFGH")
	assert plate.cols == list(range(1,13))
	
	for channel,offset in [ ("OD:600",108), ("485,528",159), ("530,580",228) ]:
		times = plate.times[channel]
		assert times[0] == offset
		assert times[-1] == offset+20*60
		assert np.all( np.diff(times)==5*60 )
	
	if temperature_ts:
		for channel in plate.channels:
			temps = plate.temperatures[channel]
			assert len(plate.times[channel]) == len(temps)
			assert all( temp==30.0 for temp in temps )
	assert plate.temperature_range == (30.0,30.0)
	
	assert plate["B12" ,"485,528"][2] == 48
	assert plate["B",12,"485,528"][2] == 48
	assert plate["b12" ,"485,528"][2] == 48
	
	assert plate["H",12,"530,580"][4] == 28
	assert plate["C",2,"OD:600"][4] == 0.085

@mark.parametrize(
		"filename",
	[
		"matrix.txt",
		"matrix_grouped.txt",
		"row.txt",
		"row_grouped.txt",
		"column.txt",
		"column_grouped.txt",
	])
def test_time_series(filename):
	data = SynergyFile(path.join("single_measurement",filename))
	assert len(data)==1
	plate = data[0]
	
	assert plate.metadata["Software Version"] == (3,2,1)
	assert plate.metadata["Experiment File Path"] == r"C:\foo.xpt"
	assert plate.metadata["Protocol File Path"] == r"C:\bar.prt"
	assert plate.metadata["Plate Number"] == "Plate 1"
	assert plate.metadata["Reader Type"] == "Synergy H1"
	assert plate.metadata["Reader Serial Number"] == "18092726"
	assert plate.metadata["Reading Type"] == "Reader"
	assert plate.metadata["procedure"] == "foo\nbar\nquz"
	
	assert plate.temperature_range == (30.0,30.1)
	
	assert plate.metadata["datetime"] == datetime(2020,7,24,15,55,48)
	
	assert plate.channels == ["Abs:600","Abs:700","Fluo:485,528"]
	assert plate.rows == list("ABCDEFGH")
	assert plate.cols == list(range(1,13))
	
	assert plate["B5","Abs:600"] == 0.101
	assert plate["D6","Abs:700"] == 0.097
	assert plate["G11","Fluo:485,528"] == 104

def test_multiple_with_gain():
	data = SynergyFile("multiple_with_gain.txt")
	assert len(data)==2
	
	for plate in data:
		assert plate.metadata["Software Version"] == (3,3,14)
		assert plate.metadata["Experiment File Path"] == r"C:\foo.xpt"
		assert plate.metadata["Protocol File Path"] == r"C:\bar.prt"
		assert plate.metadata["Plate Number"] == "Plate 1"
		assert plate.metadata["Reader Type"] == "Synergy H1"
		assert plate.metadata["Reader Serial Number"] == "14112519"
		assert plate.metadata["Reading Type"] == "Reader"
		assert plate.rows == list("ABCDEFGH")
		assert plate.cols == list(range(1,13))
	
	assert data[0].metadata["procedure"] == "foo\nbar\nquz"
	assert data[1].metadata["procedure"] == "foo\tbar\nquz"
	
	assert data[0].temperature_range == (23.8,23.8)
	assert data[1].temperature_range == (23.9,23.9)
	
	assert data[0].metadata["datetime"] == datetime(2019,12,8,16,22,44)
	assert data[1].metadata["datetime"] == datetime(2019,12,8,16,31,10)
	
	assert data[0].channels == ["YFP:485,530","CFP:360/40,460/40"]
	assert data[1].channels == ["YFP:485,530","CFP:425,475"]
	
	assert data[0].gains == {}
	assert data[1].gains["YFP:485,530"] == 134
	assert data[1].gains["CFP:425,475"] ==  74

	assert data[0]["G3","YFP:485,530"] == 553
	assert data[1]["G3","CFP:425,475"] == 1566
	assert np.isnan( data[1]["G11","YFP:485,530"] )

def test_different_format():
	plate = SynergyFile("different_format.txt")[0]
	
	assert plate.metadata["Software Version"] == (3,3,14)
	assert plate.metadata["Experiment File Path"] == r"C:\foo.xpt"
	assert plate.metadata["Protocol File Path"] == r"C:\bar.prt"
	assert plate.metadata["Plate Number"] == "Plate 1"
	assert plate.metadata["Reader Type"] == "Synergy H1"
	assert plate.metadata["Reader Serial Number"] == "14112519"
	assert plate.metadata["Reading Type"] == "Reader"
	assert plate.metadata["procedure"] == "foo\nbar\nquz"
	
	assert plate.metadata["datetime"] == datetime(2020,8,26,15,26,39)
	
	channel = "OD:600"
	assert plate.channels == [channel]
	assert plate.rows == list("ABCDEFGH")
	assert plate.cols == list(range(1,13))
	
	times = plate.times[channel]
	assert times[0] == 550
	assert np.all( np.diff(times)==10*60 )
	
	assert plate.temperatures[channel][0] == 30.0
	assert plate.temperatures[channel][90] == 30.2
	
	assert plate["A1",channel][  0] == 0.091
	assert plate["D7",channel][126] == 1.442





from ._synergy_file_reader import SynergyFile, SynergyPlate, SynergyResult

try:
	from .version import version as __version__
except ImportError:
	from warnings import warn
	warn('Failed to find (autogenerated) version.py. Do not worry about this unless you really need to know the version.')



#!/usr/bin/python3
# -*- coding: utf-8 -*-

"""
Our data is located in the file `example_data.txt` in the same folder. We can load it like this:

.. literalinclude:: ../examples/example.py
	:start-after: example-st\u0061rt
	:dedent: 1
	:lines: 1-2

Now `my_file` is (for all practical purposes) a list containing the individual plates in the file. In our case there is only one such plate, so it’s best to assign it to a seperate variable:

.. literalinclude:: ../examples/example.py
	:start-after: example-st\u0061rt
	:dedent: 1
	:lines: 4

In such a case, it often makes sense to load the file and extract the plate in one command. The following is equivalent to the above:

.. literalinclude:: ../examples/example.py
	:start-after: example-st\u0061rt
	:dedent: 1
	:lines: 1,5

If you are familiar with Python data structures, everything else is straightforward now: Have a look at the properties and keys of `my_plate` and it should be clear how to access the information you need. If not, we proceed with a small tour:

`my_plate` has a series of properties containing information we may care about. Let’s start with `channels`:

.. literalinclude:: ../examples/example.py
	:start-after: example-st\u0061rt
	:dedent: 1
	:lines: 7

This tells us that we measured only one channel for our plate, which we called *OD:600.* We can now extract and print the times and temperatures for these measurements:

.. literalinclude:: ../examples/example.py
	:start-after: example-st\u0061rt
	:dedent: 1
	:lines: 8-11

The `my_plate.metadata` is a dictionary, which contains all sorts of tangential data for our plate, as long as it is contained in the file. For example, if we want to know the version of the software that produced our plate, we can do this as follows:

.. literalinclude:: ../examples/example.py
	:start-after: example-st\u0061rt
	:dedent: 1
	:lines: 12

However, what we usually care about is the rare data. For each well and channel, this is stored in a NumPy array. We can access it by directly indexing `my_plate`. The following commands are all equivalent, with the last only working because we only have one channel:

.. literalinclude:: ../examples/example.py
	:start-after: example-st\u0061rt
	:dedent: 1
	:lines: 14-17

Our file includes no aggregated results such as growth rates computed by the Synergy software. If it did, they would be in the dictionary `my_data.results` each value of this result is a `SynergyResult` which can be indexed like `my_plate`.

If we want to access the raw data for all wells, we can do this with the `values` function which mirrors this functionality for dicitonaries. `keys` analogously gives us all well–channel combinations. In the following example, we take the first values from the time series for all wells, and compute the 10\ :sup:`th` percentile of them as a baseline for our measurements:

.. literalinclude:: ../examples/example.py
	:start-after: example-st\u0061rt
	:dedent: 1
	:lines: 19-20

The attributes `rows` and `cols` facilitate easy iterations over all rows or columns, respectively. We here compute the element-wise median of all time series in the first row (“A”), to use as a reference for comparisons:

.. literalinclude:: ../examples/example.py
	:start-after: example-st\u0061rt
	:dedent: 1
	:lines: 21

Finally, we `plot` our data. We here specify that the `baseline` we computed before shall be subtracted from all time series. Moreover, our `reference` should appear in each plot:

.. literalinclude:: ../examples/example.py
	:start-after: example-st\u0061rt
	:dedent: 1
	:lines: 23-30

The remaining arguments are hopefully self-explanatory and also explained in the documentation of `plot`. `fig` is a Matplotlib figure object with all the functionality that comes with that. For example, we can use `savefig` to export our plot to a file:

.. literalinclude:: ../examples/example.py
	:start-after: example-st\u0061rt
	:dedent: 1
	:lines: 31

And this is what our plot looks like:

.. plot:: ../examples/example.py
"""

if __name__ == "__main__":
	# example-start
	from synergy_file_reader import SynergyFile
	my_file = SynergyFile("example_data.txt")
	
	my_plate = my_file[0]
	my_plate = SynergyFile("example_data.txt")[0]
	
	print(my_plate.channels) # ['OD:600']
	channel = my_plate.channels[0]
	print(channel) # 'OD:600'
	print(my_plate.times[channel])
	print(my_plate.temperatures[channel])
	print(my_plate.metadata["Software Version"])
	
	print(my_plate["C3",channel])
	print(my_plate["c3",channel])
	print(my_plate["C",3,channel])
	print(my_plate["C3"])
	
	import numpy as np
	baseline = np.percentile( [ts[0] for ts in my_plate.values()], 10 )
	reference = np.median( [my_plate["A",col] for col in my_plate.cols], axis=0 )
	
	fig,axess = my_plate.plot(
			colours = ["red"],
			ylim = (1e-3,2), xlim = (0,30000),
			baseline = baseline,
			plot_args = {"linewidth":3},
			reference = reference,
			reference_plot_args = { "color":"black", "linewidth":1, "label":"no drug" },
		)
	fig.savefig( "example.pdf", bbox_inches="tight" )


